@@ -9,7 +9,14 @@
 from typing import cast
 
 
-def number_for_command(number: int = None, count: int = None, *, max_count: int) -> int:
+def number_for_command(
+    number: int = None,
+    count: int = None,
+    *,
+    max_count: int,
+    number_name: str = "index",
+    elem_name: str = "element",
+) -> int:
     """Return correct number for command given number, optional count and a maximum.
 
     Count is preferred over the number if it is given. The command expects numbers
@@ -17,7 +24,9 @@ def number_for_command(number: int = None, count: int = None, *, max_count: int)
     maximum, the modulo operator is used to reduce it accordingly.
     """
     if number is None and count is None:
-        raise ValueError("Either number or count must be given")
+        raise ValueError(f"Either {number_name} or count is required")
+    if max_count <= 0:
+        raise ValueError(f"No {elem_name} in list")
     if count is not None:
         number = count
     number = cast(int, number)  # Ensured by the two tests above
