@@ -6,9 +6,9 @@
 
 """Library widget with model and delegate."""
 
+import contextlib
 import os
-from contextlib import suppress
-from typing import List, Optional, Dict
+from typing import List, Optional, Dict, NamedTuple
 
 from PyQt5.QtCore import Qt, pyqtSlot
 from PyQt5.QtWidgets import QStyledItemDelegate, QSizePolicy, QStyle
@@ -17,13 +17,25 @@ from PyQt5.QtGui import QStandardItemModel, QColor, QTextDocument, QStandardItem
 from vimiv import api, utils, widgets
 from vimiv.commands import argtypes, search, number_for_command
 from vimiv.config import styles
+from vimiv.gui import eventhandler, synchronize
 from vimiv.utils import files, strip_html, clamp, wrap_style_span, log
-from . import eventhandler, synchronize
 
 
 _logger = log.module_logger(__name__)
 
 
+class Position(NamedTuple):
+    """Storage class for a position in the library.
+
+    Attributes:
+        path: Path of the stored position.
+        row: Row of the stored position used in case the path is no longer available.
+    """
+
+    path: str
+    row: int = 0
+
+
 class Library(eventhandler.EventHandlerMixin, widgets.FlatTreeView):
     """Library widget.
 
@@ -72,7 +84,7 @@ class Library(eventhandler.EventHandlerMixin, widgets.FlatTreeView):
     def __init__(self, mainwindow):
         super().__init__(parent=mainwindow)
         self._last_selected = ""
-        self._positions: Dict[str, str] = {}
+        self._positions: Dict[str, Position] = {}
 
         self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
         self.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Ignored)
@@ -88,7 +100,7 @@ class Library(eventhandler.EventHandlerMixin, widgets.FlatTreeView):
         search.search.new_search.connect(self._on_new_search)
         search.search.cleared.connect(self.repaint)
         api.modes.LIBRARY.entered.connect(self._on_enter)
-        api.modes.LIBRARY.left.connect(self._on_left)
+        api.modes.LIBRARY.closed.connect(self._on_closed)
         api.signals.new_image_opened.connect(self._select_path)
         synchronize.signals.new_thumbnail_path_selected.connect(self._select_path)
 
@@ -118,20 +130,19 @@ class Library(eventhandler.EventHandlerMixin, widgets.FlatTreeView):
     def _on_enter(self):
         """Update widths and ensure that the current path is visible at the center."""
         self.update_width()
-        self.scrollTo(self.currentIndex(), hint=self.PositionAtCenter)
 
     @utils.slot
-    def _on_left(self):
-        """Hide library widget if library mode was left."""
+    def _on_closed(self):
+        """Hide library widget if library mode was clsed."""
         self.hide()
         self._last_selected = ""
 
     @utils.slot
     def _select_path(self, path: str):
         """Select a specific path by name."""
-        with suppress(ValueError):
+        with contextlib.suppress(ValueError):
             _logger.debug("Selecting library path '%s' via slot", path)
-            self._select_row(self.model().paths.index(path), emit=False)  # type: ignore
+            self._select_row(self.model().paths.index(path), emit=False)
 
     @api.commands.register(mode=api.modes.LIBRARY)
     def open_selected(self, close: bool = False):
@@ -167,7 +178,7 @@ class Library(eventhandler.EventHandlerMixin, widgets.FlatTreeView):
             api.signals.load_images.emit([path])
         self._last_selected = path
         if close:
-            api.modes.LIBRARY.leave()
+            api.modes.LIBRARY.close()
 
     @api.keybindings.register("p", "scroll up --open-selected", mode=api.modes.LIBRARY)
     @api.keybindings.register("k", "scroll up", mode=api.modes.LIBRARY)
@@ -209,13 +220,11 @@ class Library(eventhandler.EventHandlerMixin, widgets.FlatTreeView):
         elif direction == direction.Left:
             self.store_position()
             parent = os.path.abspath(os.pardir)
-            self._positions[parent] = os.getcwd()
+            self._positions[parent] = Position(os.getcwd())
             api.working_directory.handler.chdir(parent)
         else:
-            try:
-                row = self.row()
-            # Directory is empty
-            except IndexError:
+            row = self.row()
+            if row == -1:  # Directory is empty
                 raise api.commands.CommandWarning("Directory is empty")
             if direction == direction.Up:
                 row -= count
@@ -248,9 +257,15 @@ class Library(eventhandler.EventHandlerMixin, widgets.FlatTreeView):
         **count:** Select [count]th element instead.
         """
         try:
-            row = number_for_command(row, count, max_count=self.model().rowCount())
-        except ValueError:
-            raise api.commands.CommandError("Either row or count is required")
+            row = number_for_command(
+                row,
+                count,
+                max_count=self.model().rowCount(),
+                number_name="row",
+                elem_name="path",
+            )
+        except ValueError as e:
+            raise api.commands.CommandError(str(e))
         self._select_row(row, open_selected_image=open_selected)
 
     def update_width(self):
@@ -262,8 +277,8 @@ class Library(eventhandler.EventHandlerMixin, widgets.FlatTreeView):
 
     def current(self):
         """Return absolute path of currently selected path."""
-        with suppress(IndexError):  # No path selected
-            basename = self.selectionModel().selectedIndexes()[1].data()
+        with contextlib.suppress(IndexError):  # No path selected
+            basename = self.selectedIndexes()[1].data()
             basename = strip(basename)
             return os.path.abspath(basename)
         return ""
@@ -274,24 +289,30 @@ class Library(eventhandler.EventHandlerMixin, widgets.FlatTreeView):
 
     def store_position(self):
         """Set the stored position for a directory if possible."""
-        with suppress(IndexError):
-            self._positions[os.getcwd()] = self.current()
+        with contextlib.suppress(IndexError):
+            self._positions[os.getcwd()] = Position(self.current(), self.row())
 
     def load_directory(self):
         """Update library for new or reloaded directory."""
         _logger.debug("Updating library for new/reloaded directory")
-        try:
-            row = self._get_stored_position()
-            _logger.debug("Retrieved stored position %d", row)
-        except (KeyError, ValueError):
-            _logger.debug("No stored position, falling back to 0")
-            row = 0
+        row = self._get_stored_position()
         self._select_row(row)
         self.update_width()
 
     def _get_stored_position(self):
-        stored_path = self._positions[os.getcwd()]
-        return self.model().paths.index(stored_path)
+        """Retrieve row of stored path with various fallback options."""
+        stored_position = self._positions.get(os.getcwd())
+        if stored_position is not None:
+            try:
+                row = self.model().paths.index(stored_position.path)
+                _logger.debug("Retrieved stored position from path: %d", row)
+                return row
+            except ValueError:  # stored_path no longer exists
+                row = min(stored_position.row, self.model().rowCount() - 1)
+                _logger.debug("Stored position longer exists, using fallback %d", row)
+                return row
+        _logger.debug("No stored position, falling back to 0")
+        return 0
 
     def _select_row(
         self, row: int, open_selected_image: bool = False, emit: bool = True
@@ -418,7 +439,7 @@ class LibraryModel(QStandardItemModel):
                 name = utils.add_html(name + "/", "b")
             if path in api.mark.paths:
                 name = mark_prefix + name
-            with suppress(FileNotFoundError):  # Has been deleted in the meantime
+            with contextlib.suppress(FileNotFoundError):  # Deleted in the meantime
                 size = get_size(path)
                 self.appendRow(
                     (QStandardItem(str(i)), QStandardItem(name), QStandardItem(size),)
