@@ -6,25 +6,27 @@
 
 """Thumbnail widget."""
 
+import contextlib
 import os
-from contextlib import suppress
 from typing import List, Optional, Iterator, cast
 
-from PyQt5.QtCore import Qt, QSize, QItemSelectionModel, QRect, pyqtSlot
+from PyQt5.QtCore import Qt, QSize, QRect, pyqtSlot
 from PyQt5.QtWidgets import QListWidget, QListWidgetItem, QStyle, QStyledItemDelegate
 from PyQt5.QtGui import QColor, QIcon
 
-from vimiv import api, utils, imutils
+from vimiv import api, utils, imutils, widgets
 from vimiv.commands import argtypes, search, number_for_command
 from vimiv.config import styles
+from vimiv.gui import eventhandler, synchronize
 from vimiv.utils import create_pixmap, thumbnail_manager, log
-from . import eventhandler, synchronize
 
 
 _logger = log.module_logger(__name__)
 
 
-class ThumbnailView(eventhandler.EventHandlerMixin, QListWidget):
+class ThumbnailView(
+    eventhandler.EventHandlerMixin, widgets.ScrollToCenterMixin, QListWidget
+):
     """Thumbnail widget.
 
     Attributes:
@@ -251,9 +253,11 @@ class ThumbnailView(eventhandler.EventHandlerMixin, QListWidget):
         **count:** Select [count]th thubnail instead.
         """
         try:
-            index = number_for_command(index, count, max_count=self.count())
-        except ValueError:
-            raise api.commands.CommandError("Either index or count is required")
+            index = number_for_command(
+                index, count, max_count=self.count(), elem_name="thumbnail"
+            )
+        except ValueError as e:
+            raise api.commands.CommandError(str(e))
         self._select_index(index)
 
     @api.keybindings.register("-", "zoom out", mode=api.modes.THUMBNAIL)
@@ -277,12 +281,12 @@ class ThumbnailView(eventhandler.EventHandlerMixin, QListWidget):
         for i in range(self.count()):
             item = self.item(i)
             item.setSizeHint(QSize(self.item_size(), self.item_size()))
-        self.scrollTo(self.selectionModel().currentIndex(), hint=self.PositionAtCenter)
+        self.scrollTo(self.currentIndex())
 
     @utils.slot
     def _select_path(self, path: str):
         """Select a specific path by name."""
-        with suppress(ValueError):
+        with contextlib.suppress(ValueError):
             self._select_index(self._paths.index(path), emit=False)
 
     def _select_index(self, index: int, emit: bool = True) -> None:
@@ -292,11 +296,11 @@ class ThumbnailView(eventhandler.EventHandlerMixin, QListWidget):
             index: Number of the current item to select.
             emit: Emit the new_thumbnail_path_selected signal.
         """
+        if not self._paths:
+            raise api.commands.CommandWarning("Thumbnail list is empty")
         _logger.debug("Selecting thumbnail number %d", index)
         model_index = self.model().index(index, 0)
-        selmod = QItemSelectionModel.Rows | QItemSelectionModel.ClearAndSelect
-        self.selectionModel().setCurrentIndex(model_index, selmod)  # type: ignore
-        self.scrollTo(model_index, hint=self.PositionAtCenter)
+        self.setCurrentIndex(model_index)
         if emit:
             synchronize.signals.new_thumbnail_path_selected.emit(self._paths[index])
 
@@ -357,7 +361,7 @@ class ThumbnailView(eventhandler.EventHandlerMixin, QListWidget):
     def resizeEvent(self, event):
         """Update resize event to keep selected thumbnail centered."""
         super().resizeEvent(event)
-        self.scrollTo(self.selectionModel().currentIndex(), hint=self.PositionAtCenter)
+        self.scrollTo(self.currentIndex())
 
 
 class ThumbnailDelegate(QStyledItemDelegate):
@@ -482,22 +486,30 @@ class ThumbnailItem(QListWidgetItem):
 
     _default_icon = None
 
-    def __init__(self, parent, index, highlighted=False, marked=False, size_hint=None):
-        super().__init__(self.default_icon, "", parent, index)
+    def __init__(self, parent, index, *, size_hint, highlighted=False, marked=False):
+        super().__init__(self.default_icon(), "", parent, index)
         self.highlighted = highlighted
         self.marked = marked
         self.setSizeHint(size_hint)
 
-    @property
-    def default_icon(self):
-        """Default icon if the thumbnail has not been created."""
-        if self._default_icon is None:
-            self._default_icon = QIcon(
-                create_pixmap(
-                    color=styles.get("thumbnail.default.bg"),
-                    frame_color=styles.get("thumbnail.frame.fg"),
-                    size=256,
-                    frame_size=10,
-                )
+    @classmethod
+    def default_icon(cls):
+        """Default icon if the thumbnail has not been created.
+
+        The return value is stored to avoid re-creating the pixmap for every thumbnail.
+        """
+        if cls._default_icon is None:
+            cls._default_icon = cls.create_default_icon()
+        return cls._default_icon
+
+    @classmethod
+    def create_default_icon(cls):
+        """Create the default icon shown if the thumbnail has not been created."""
+        return QIcon(
+            create_pixmap(
+                color=styles.get("thumbnail.default.bg"),
+                frame_color=styles.get("thumbnail.frame.fg"),
+                size=256,
+                frame_size=10,
             )
-        return self._default_icon
+        )
